@@ -1,5 +1,5 @@
 /*
- * Copyright © 2013 Canonical Ltd.
+ * Copyright © 2013-2015 Canonical Ltd.
  *
  * This program is free software: you can redistribute it and/or modify it
  * under the terms of the GNU Lesser General Public License version 3,
@@ -17,24 +17,38 @@
  */
 
 #include <core/media/gstreamer/playbin.h>
-
 #include <core/media/gstreamer/engine.h>
 
+#include <gst/pbutils/missing-plugins.h>
+
 #if defined(MEDIA_HUB_HAVE_HYBRIS_MEDIA_COMPAT_LAYER)
 #include <hybris/media/surface_texture_client_hybris.h>
 #include <hybris/media/media_codec_layer.h>
 
+#include "../util/uri_check.h"
+
+#include <utility>
+
+//#define VERBOSE_DEBUG
+
 namespace
 {
-void setup_video_sink_for_buffer_streaming(GstElement* video_sink)
+void setup_video_sink_for_buffer_streaming(GstElement* pipeline)
 {
     // Get the service-side BufferQueue (IGraphicBufferProducer) and associate it with
     // the SurfaceTextureClientHybris instance
     IGBPWrapperHybris igbp = decoding_service_get_igraphicbufferproducer();
     SurfaceTextureClientHybris stc = surface_texture_client_create_by_igbp(igbp);
+
     // Because mirsink is being loaded, we are definitely doing * hardware rendering.
-    surface_texture_client_set_hardware_rendering (stc, TRUE);
-    g_object_set (G_OBJECT (video_sink), "surface", static_cast<gpointer>(stc), static_cast<char*>(NULL));
+    surface_texture_client_set_hardware_rendering(stc, TRUE);
+
+    GstContext *context = gst_context_new("gst.mir.MirContext", TRUE);
+    GstStructure *structure = gst_context_writable_structure(context);
+    gst_structure_set(structure, "gst_mir_context", G_TYPE_POINTER, stc, NULL);
+
+    /* Propagate context in pipeline (needed by amchybris and mirsink) */
+    gst_element_set_context(pipeline, context);
 }
 }
 #else  // MEDIA_HUB_HAVE_HYBRIS_MEDIA_COMPAT_LAYER
@@ -42,7 +56,7 @@ namespace
 {
 void setup_video_sink_for_buffer_streaming(GstElement*)
 {
-    throw core::ubuntu::media::Player::Error::OutOfProcessBufferStreamingNotSupported{};
+    throw core::ubuntu::media::Player::Errors::OutOfProcessBufferStreamingNotSupported{};
 }
 }
 #endif // MEDIA_HUB_HAVE_HYBRIS_MEDIA_COMPAT_LAYER
@@ -90,34 +104,41 @@ gstreamer::Playbin::Playbin()
       bus{gst_element_get_bus(pipeline)},
       file_type(MEDIA_FILE_TYPE_NONE),
       video_sink(nullptr),
-      on_new_message_connection(
-          bus.on_new_message.connect(
+      audio_sink(nullptr),
+      on_new_message_connection_async(
+          bus.on_new_message_async.connect(
               std::bind(
-                  &Playbin::on_new_message,
+                  &Playbin::on_new_message_async,
                   this,
                   std::placeholders::_1))),
       is_seeking(false),
       previous_position(0),
+      cached_video_dimensions{
+        core::ubuntu::media::video::Height{0},
+        core::ubuntu::media::video::Width{0}},
       player_lifetime(media::Player::Lifetime::normal),
-      is_eos(false)
+      about_to_finish_handler_id(0),
+      source_setup_handler_id(0),
+      is_missing_audio_codec(false),
+      is_missing_video_codec(false),
+      audio_stream_id(-1),
+      video_stream_id(-1)
 {
     if (!pipeline)
         throw std::runtime_error("Could not create pipeline for playbin.");
 
-    is_eos = false;
-
     // Add audio and/or video sink elements depending on environment variables
     // being set or not set
     setup_pipeline_for_audio_video();
 
-    g_signal_connect(
+    about_to_finish_handler_id = g_signal_connect(
                 pipeline,
                 "about-to-finish",
                 G_CALLBACK(about_to_finish),
                 this
                 );
 
-    g_signal_connect(
+    source_setup_handler_id = g_signal_connect(
         pipeline,
         "source-setup",
         G_CALLBACK(source_setup),
@@ -125,10 +146,39 @@ gstreamer::Playbin::Playbin()
         );
 }
 
+// Note that we might be accessing freed memory here, so activate DEBUG_REFS
+// only for debugging
+//#define DEBUG_REFS
+#ifdef DEBUG_REFS
+static void print_refs(const gstreamer::Playbin &pb, const char *func)
+{
+    using namespace std;
+
+    cout << func << endl;
+    if (pb.pipeline)
+        cout << "pipeline:   " << GST_OBJECT_REFCOUNT(pb.pipeline) << endl;
+    if (pb.video_sink)
+        cout << "video_sink: " << GST_OBJECT_REFCOUNT(pb.video_sink) << endl;
+    if (pb.audio_sink)
+        cout << "audio_sink: " << GST_OBJECT_REFCOUNT(pb.audio_sink) << endl;
+}
+#endif
+
 gstreamer::Playbin::~Playbin()
 {
+#ifdef DEBUG_REFS
+    print_refs(*this, "gstreamer::Playbin::~Playbin pipeline");
+#endif
+
+    g_signal_handler_disconnect(pipeline, about_to_finish_handler_id);
+    g_signal_handler_disconnect(pipeline, source_setup_handler_id);
+
     if (pipeline)
         gst_object_unref(pipeline);
+
+#ifdef DEBUG_REFS
+    print_refs(*this, "gstreamer::Playbin::~Playbin pipeline");
+#endif
 }
 
 void gstreamer::Playbin::reset()
@@ -163,9 +213,47 @@ void gstreamer::Playbin::reset_pipeline()
         std::cout << "Failed to reset the pipeline state. Client reconnect may not function properly." << std::endl;
     }
     file_type = MEDIA_FILE_TYPE_NONE;
+    is_missing_audio_codec = false;
+    is_missing_video_codec = false;
+    audio_stream_id = -1;
+    video_stream_id = -1;
 }
 
-void gstreamer::Playbin::on_new_message(const Bus::Message& message)
+void gstreamer::Playbin::process_message_element(GstMessage *message)
+{
+    if (!gst_is_missing_plugin_message(message))
+      return;
+
+    gchar *desc = gst_missing_plugin_message_get_description(message);
+    std::cerr << "Missing plugin: " << desc << std::endl;
+    g_free(desc);
+
+    const GstStructure *msg_data = gst_message_get_structure(message);
+    if (g_strcmp0("decoder", gst_structure_get_string(msg_data, "type")) != 0)
+        return;
+
+    GstCaps *caps;
+    if (!gst_structure_get(msg_data, "detail", GST_TYPE_CAPS, &caps, NULL)) {
+        std::cerr << __PRETTY_FUNCTION__ << ": No detail" << std::endl;
+        return;
+    }
+
+    GstStructure *caps_data = gst_caps_get_structure(caps, 0);
+    if (!caps_data) {
+        std::cerr << __PRETTY_FUNCTION__ << ": No caps data" << std::endl;
+        return;
+    }
+
+    const gchar *mime = gst_structure_get_name(caps_data);
+    if (strstr(mime, "audio"))
+        is_missing_audio_codec = true;
+    else if (strstr(mime, "video"))
+        is_missing_video_codec = true;
+
+    std::cerr << "Missing decoder for " << mime << std::endl;
+}
+
+void gstreamer::Playbin::on_new_message_async(const Bus::Message& message)
 {
     switch(message.type)
     {
@@ -178,6 +266,16 @@ void gstreamer::Playbin::on_new_message(const Bus::Message& message)
     case GST_MESSAGE_INFO:
         signals.on_info(message.detail.error_warning_info);
         break;
+    case GST_MESSAGE_STATE_CHANGED:
+        if (message.source == "playbin") {
+            g_object_get(G_OBJECT(pipeline), "current-audio", &audio_stream_id, NULL);
+            g_object_get(G_OBJECT(pipeline), "current-video", &video_stream_id, NULL);
+        }
+        signals.on_state_changed(std::make_pair(message.detail.state_changed, message.source));
+        break;
+    case GST_MESSAGE_ELEMENT:
+        process_message_element(message.message);
+        break;
     case GST_MESSAGE_TAG:
         {
             gchar *orientation;
@@ -191,9 +289,6 @@ void gstreamer::Playbin::on_new_message(const Bus::Message& message)
             signals.on_tag_available(message.detail.tag);
         }
         break;
-    case GST_MESSAGE_STATE_CHANGED:
-        signals.on_state_changed(message.detail.state_changed);
-        break;
     case GST_MESSAGE_ASYNC_DONE:
         if (is_seeking)
         {
@@ -203,7 +298,6 @@ void gstreamer::Playbin::on_new_message(const Bus::Message& message)
         }
         break;
     case GST_MESSAGE_EOS:
-        is_eos = true;
         signals.on_end_of_stream();
     default:
         break;
@@ -226,7 +320,7 @@ void gstreamer::Playbin::setup_pipeline_for_audio_video()
 
     if (::getenv("CORE_UBUNTU_MEDIA_SERVICE_AUDIO_SINK_NAME") != nullptr)
     {
-        auto audio_sink = gst_element_factory_make (
+        audio_sink = gst_element_factory_make (
                     ::getenv("CORE_UBUNTU_MEDIA_SERVICE_AUDIO_SINK_NAME"),
                     "audio-sink");
 
@@ -262,7 +356,7 @@ void gstreamer::Playbin::create_video_sink(uint32_t)
         "No video sink configured for the current pipeline"
     };
 
-    setup_video_sink_for_buffer_streaming(video_sink);
+    setup_video_sink_for_buffer_streaming(pipeline);
 }
 
 void gstreamer::Playbin::set_volume(double new_volume)
@@ -310,17 +404,13 @@ media::Player::Orientation gstreamer::Playbin::orientation_lut(const gchar *orie
 /** Sets the new audio stream role on the pulsesink in playbin */
 void gstreamer::Playbin::set_audio_stream_role(media::Player::AudioStreamRole new_audio_role)
 {
-    GstElement *audio_sink = NULL;
-    g_object_get (pipeline, "audio-sink", &audio_sink, NULL);
-
     std::string role_str("props,media.role=" + get_audio_role_str(new_audio_role));
     std::cout << "Audio stream role: " << role_str << std::endl;
 
     GstStructure *props = gst_structure_from_string (role_str.c_str(), NULL);
-    if (audio_sink != nullptr && props != nullptr)
+    if (audio_sink != nullptr && props != nullptr) {
         g_object_set (audio_sink, "stream-properties", props, NULL);
-    else
-    {
+    } else {
         std::cerr <<
             "Warning: couldn't set audio stream role - couldn't get audio_sink from pipeline" <<
             std::endl;
@@ -365,17 +455,42 @@ uint64_t gstreamer::Playbin::duration() const
 
 void gstreamer::Playbin::set_uri(
     const std::string& uri,
-    const core::ubuntu::media::Player::HeadersType& headers = core::ubuntu::media::Player::HeadersType())
+    const core::ubuntu::media::Player::HeadersType& headers = core::ubuntu::media::Player::HeadersType(),
+    bool do_pipeline_reset)
 {
-    reset_pipeline();
+    gchar *current_uri = nullptr;
+    g_object_get(pipeline, "current-uri", &current_uri, NULL);
 
-    g_object_set(pipeline, "uri", uri.c_str(), NULL);
-    if (is_video_file(uri))
+    // Checking for a current_uri being set and not resetting the pipeline
+    // if there isn't a current_uri causes the first play to start playback
+    // sooner since reset_pipeline won't be called
+    if (current_uri and do_pipeline_reset)
+        reset_pipeline();
+
+    std::string tmp_uri{uri};
+    media::UriCheck::Ptr uri_check{std::make_shared<media::UriCheck>(uri)};
+    if (uri_check->is_local_file())
+    {
+        if (uri_check->is_encoded())
+        {
+            // First decode the URI just in case it's partially encoded already
+            tmp_uri = decode_uri(uri);
+#ifdef VERBOSE_DEBUG
+            std::cout << "File URI was encoded, now decoded: " << tmp_uri << std::endl;
+#endif
+        }
+        tmp_uri = encode_uri(tmp_uri);
+    }
+
+    g_object_set(pipeline, "uri", tmp_uri.c_str(), NULL);
+    if (is_video_file(tmp_uri))
         file_type = MEDIA_FILE_TYPE_VIDEO;
-    else if (is_audio_file(uri))
+    else if (is_audio_file(tmp_uri))
         file_type = MEDIA_FILE_TYPE_AUDIO;
 
     request_headers = headers;
+
+    g_free(current_uri);
 }
 
 void gstreamer::Playbin::setup_source(GstElement *source)
@@ -443,14 +558,15 @@ bool gstreamer::Playbin::set_state_and_wait(GstState new_state)
     }
 
     // We only should query the pipeline if we actually succeeded in
-    // setting the requested state. Also don't send on_video_dimensions_changed
-    // signal during EOS.
-    if (result && new_state == GST_STATE_PLAYING && !is_eos)
+    // setting the requested state.
+    if (result && new_state == GST_STATE_PLAYING)
     {
         // Get the video height/width from the video sink
         try
         {
-            signals.on_video_dimensions_changed(get_video_dimensions());
+            const core::ubuntu::media::video::Dimensions new_dimensions = get_video_dimensions();
+            emit_video_dimensions_changed_if_changed(new_dimensions);
+            cached_video_dimensions = new_dimensions;
         }
         catch (const std::exception& e)
         {
@@ -492,6 +608,7 @@ core::ubuntu::media::video::Dimensions gstreamer::Playbin::get_video_dimensions(
     uint32_t video_width = 0, video_height = 0;
     g_object_get (video_sink, "height", &video_height, nullptr);
     g_object_get (video_sink, "width", &video_width, nullptr);
+
     // TODO(tvoss): We should probably check here if width and height are valid.
     return core::ubuntu::media::video::Dimensions
     {
@@ -500,26 +617,21 @@ core::ubuntu::media::video::Dimensions gstreamer::Playbin::get_video_dimensions(
     };
 }
 
-std::string gstreamer::Playbin::get_file_content_type(const std::string& uri) const
+void gstreamer::Playbin::emit_video_dimensions_changed_if_changed(const core::ubuntu::media::video::Dimensions &new_dimensions)
 {
-    if (uri.empty())
-        return std::string();
-
-    std::string filename(uri);
-    std::cout << "filename: " << filename << std::endl;
-    size_t pos = uri.find("file://");
-    if (pos != std::string::npos)
-        filename = uri.substr(pos + 7, std::string::npos);
-    else
-        // Anything other than a file, for now claim that the type
-        // is both audio and video.
-        // FIXME: implement true net stream sampling and get the type from GstCaps
-        return std::string("audio/video/");
-
+    // Only signal the application layer if the dimensions have in fact changed. This might happen
+    // if reusing the same media-hub session to play two different video sources.
+    if (new_dimensions != cached_video_dimensions)
+        signals.on_video_dimensions_changed(new_dimensions);
+}
 
+std::string gstreamer::Playbin::file_info_from_uri(const std::string& uri) const
+{
     GError *error = nullptr;
+    // Open the URI and get the mime type from it. This will currently only work for
+    // a local file
     std::unique_ptr<GFile, void(*)(void *)> file(
-            g_file_new_for_path(filename.c_str()), g_object_unref);
+            g_file_new_for_uri(uri.c_str()), g_object_unref);
     std::unique_ptr<GFileInfo, void(*)(void *)> info(
             g_file_query_info(
                 file.get(), G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE ","
@@ -527,17 +639,114 @@ std::string gstreamer::Playbin::get_file_content_type(const std::string& uri) co
                 /* cancellable */ NULL, &error),
             g_object_unref);
     if (!info)
-    {
-        std::string error_str(error->message);
-        g_error_free(error);
-
-        std::cout << "Failed to query the URI for the presence of video content: "
-            << error_str << std::endl;
         return std::string();
+
+    return std::string(g_file_info_get_attribute_string(
+                info.get(), G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE));
+}
+
+std::string gstreamer::Playbin::encode_uri(const std::string& uri) const
+{
+    if (uri.empty())
+        return std::string();
+
+    std::string encoded_uri;
+    media::UriCheck::Ptr uri_check{std::make_shared<media::UriCheck>(uri)};
+    gchar *uri_scheme = g_uri_parse_scheme(uri.c_str());
+    // We have a URI and it is already percent encoded
+    if (uri_scheme and strlen(uri_scheme) > 0 and uri_check->is_encoded())
+    {
+#ifdef VERBOSE_DEBUG
+        std::cout << "Is a URI and is already percent encoded" << std::endl;
+#endif
+        encoded_uri = uri;
+    }
+    // We have a URI but it's not already percent encoded
+    else if (uri_scheme and strlen(uri_scheme) > 0 and !uri_check->is_encoded())
+    {
+#ifdef VERBOSE_DEBUG
+        std::cout << "Is a URI and is not already percent encoded" << std::endl;
+#endif
+        gchar *encoded = g_uri_escape_string(uri.c_str(),
+                                                   "!$&'()*+,;=:/?[]@", // reserved chars
+                                                   TRUE); // Allow UTF-8 chars
+        if (!encoded)
+        {
+            g_free(uri_scheme);
+            return std::string();
+        }
+        encoded_uri.assign(encoded);
+        g_free(encoded);
+    }
+    else // We have a path and not a URI. Turn it into a full URI and encode it
+    {
+        GError *error = nullptr;
+#ifdef VERBOSE_DEBUG
+        std::cout << "Is a path and is not already percent encoded" << std::endl;
+#endif
+        gchar *str = g_filename_to_uri(uri.c_str(), nullptr, &error);
+        if (!str)
+        {
+            g_free(uri_scheme);
+            return std::string();
+        }
+        encoded_uri.assign(str);
+        g_free(str);
+        if (error != nullptr)
+        {
+            std::cerr << "Warning: failed to get actual track content type: " << error->message
+                      << std::endl;
+            g_error_free(error);
+            g_free(str);
+            g_free(uri_scheme);
+            return std::string("audio/video/");
+        }
+        gchar *escaped = g_uri_escape_string(encoded_uri.c_str(),
+                                         "!$&'()*+,;=:/?[]@", // reserved chars
+                                         TRUE); // Allow UTF-8 chars
+        if (!escaped)
+        {
+            g_free(uri_scheme);
+            return std::string();
+        }
+        encoded_uri.assign(escaped);
+        g_free(escaped);
     }
 
-    std::string content_type(g_file_info_get_attribute_string(
-                info.get(), G_FILE_ATTRIBUTE_STANDARD_FAST_CONTENT_TYPE));
+    g_free(uri_scheme);
+
+    return encoded_uri;
+}
+
+std::string gstreamer::Playbin::decode_uri(const std::string& uri) const
+{
+    if (uri.empty())
+        return std::string();
+
+    gchar *decoded_gchar = g_uri_unescape_string(uri.c_str(), nullptr);
+    if (!decoded_gchar)
+        return std::string();
+
+    const std::string decoded{decoded_gchar};
+    g_free(decoded_gchar);
+    return decoded;
+}
+
+std::string gstreamer::Playbin::get_file_content_type(const std::string& uri) const
+{
+    if (uri.empty())
+        return std::string();
+
+    const std::string encoded_uri{encode_uri(uri)};
+
+    const std::string content_type {file_info_from_uri(encoded_uri)};
+    if (content_type.empty())
+    {
+        std::cerr << "Warning: failed to get actual track content type" << std::endl;
+        return std::string("audio/video/");
+    }
+
+    std::cout << "Found content type: " << content_type << std::endl;
 
     return content_type;
 }
@@ -574,3 +783,22 @@ gstreamer::Playbin::MediaFileType gstreamer::Playbin::media_file_type() const
 {
     return file_type;
 }
+
+bool gstreamer::Playbin::can_play_streams() const
+{
+    /*
+     * We do not consider that we can play the video when
+     * 1. No audio stream selected due to missing decoder
+     * 2. No video stream selected due to missing decoder
+     * 3. No stream selected at all
+     * Note that if there are several, say, audio streams, we will play the file
+     * provided that we can decode just one of them, even if there are missing
+     * audio codecs. We will also play files with only one type of stream.
+     */
+    if ((is_missing_audio_codec && audio_stream_id == -1) ||
+            (is_missing_video_codec && video_stream_id == -1) ||
+            (audio_stream_id == -1 && video_stream_id == -1))
+        return false;
+    else
+        return true;
+}
