@@ -71,6 +71,8 @@ struct Playbin
     void reset_pipeline();
 
     void on_new_message(const Bus::Message& message);
+    void on_new_message_async(const Bus::Message& message);
+    void process_message_element(GstMessage *message);
 
     gstreamer::Bus& message_bus();
 
@@ -89,16 +91,22 @@ struct Playbin
     uint64_t position() const;
     uint64_t duration() const;
 
-    void set_uri(const std::string& uri, const core::ubuntu::media::Player::HeadersType& headers);
+    void set_uri(const std::string& uri, const core::ubuntu::media::Player::HeadersType& headers, bool do_pipeline_reset = true);
     std::string uri() const;
 
     void setup_source(GstElement *source);
 
+    // Sets the pipeline's state (stopped, playing, paused, etc). Optional parameter makes this call
+    // in the main_loop context.
     bool set_state_and_wait(GstState new_state);
     bool seek(const std::chrono::microseconds& ms);
 
     core::ubuntu::media::video::Dimensions get_video_dimensions() const;
+    void emit_video_dimensions_changed_if_changed(const core::ubuntu::media::video::Dimensions &new_dimensions);
 
+    std::string file_info_from_uri(const std::string& uri) const;
+    std::string encode_uri(const std::string& uri) const;
+    std::string decode_uri(const std::string& uri) const;
     std::string get_file_content_type(const std::string& uri) const;
 
     bool is_audio_file(const std::string& uri) const;
@@ -106,16 +114,21 @@ struct Playbin
 
     MediaFileType media_file_type() const;
 
+    bool can_play_streams() const;
+
     GstElement* pipeline;
     gstreamer::Bus bus;
     MediaFileType file_type;
     GstElement* video_sink;
-    core::Connection on_new_message_connection;
+    GstElement* audio_sink;
+    core::Connection on_new_message_connection_async;
     bool is_seeking;
     mutable uint64_t previous_position;
+    core::ubuntu::media::video::Dimensions cached_video_dimensions;
     core::ubuntu::media::Player::HeadersType request_headers;
     core::ubuntu::media::Player::Lifetime player_lifetime;
-    bool is_eos;
+    gulong about_to_finish_handler_id;
+    gulong source_setup_handler_id;
     struct
     {
         core::Signal<void> about_to_finish;
@@ -123,7 +136,7 @@ struct Playbin
         core::Signal<Bus::Message::Detail::ErrorWarningInfo> on_warning;
         core::Signal<Bus::Message::Detail::ErrorWarningInfo> on_info;
         core::Signal<Bus::Message::Detail::Tag> on_tag_available;
-        core::Signal<Bus::Message::Detail::StateChanged> on_state_changed;
+        core::Signal<std::pair<Bus::Message::Detail::StateChanged,std::string>> on_state_changed;
         core::Signal<uint64_t> on_seeked_to;
         core::Signal<void> on_end_of_stream;
         core::Signal<core::ubuntu::media::Player::PlaybackStatus> on_playback_status_changed;
@@ -131,6 +144,10 @@ struct Playbin
         core::Signal<core::ubuntu::media::video::Dimensions> on_video_dimensions_changed;
         core::Signal<void> client_disconnected;
     } signals;
+    bool is_missing_audio_codec;
+    bool is_missing_video_codec;
+    gint audio_stream_id;
+    gint video_stream_id;
 };
 }
 
