@@ -22,11 +22,27 @@
 
 namespace media = core::ubuntu::media;
 
+media::Player::Errors::OutOfProcessBufferStreamingNotSupported::OutOfProcessBufferStreamingNotSupported()
+    : std::runtime_error{"Implementation does not support out-of-process buffer streaming"}
+{
+}
+
+media::Player::Errors::InsufficientAppArmorPermissions::InsufficientAppArmorPermissions
+    (const std::string &e)
+    : std::runtime_error{e}
+{
+}
+
+media::Player::Errors::UriNotFound::UriNotFound
+    (const std::string &e)
+    : std::runtime_error{e}
+{
+}
+
 const media::Player::Configuration& media::Player::Client::default_configuration()
 {
     static const media::Player::Configuration config
     {
-        std::string{""},
         0,
         nullptr,
         nullptr
