@@ -292,15 +292,37 @@ struct media::PlayerImplementation<Parent>::Private :
         const Track::UriType uri = track_list->query_uri_for_track(id);
         if (!uri.empty())
         {
-            // Using a TrackList for playback, added tracks via add_track(), but open_uri hasn't been called yet
-            // to load a media resource
-            std::cout << "Calling d->engine->open_resource_for_uri() for first track added only: " << uri << std::endl;
+            // Using a TrackList for playback, added tracks via add_track(), but open_uri hasn't
+            // been called yet to load a media resource
+            std::cout << "Calling d->engine->open_resource_for_uri() for first track added only: "
+                      << uri << std::endl;
             std::cout << "\twith a Track::Id: " << id << std::endl;
             static const bool do_pipeline_reset = false;
             engine->open_resource_for_uri(uri, do_pipeline_reset);
         }
     }
 
+    void update_mpris_properties(void)
+    {
+        bool has_previous =    track_list->has_previous()
+                            or parent->Parent::loop_status() != Player::LoopStatus::none;
+        bool has_next =    track_list->has_next()
+                        or parent->Parent::loop_status() != Player::LoopStatus::none;
+        auto n_tracks = track_list->tracks()->size();
+        bool has_tracks = (n_tracks > 0) ? true : false;
+
+        std::cout << "Updating MPRIS TrackList properties"
+                  << "; Tracks: " << n_tracks
+                  << ", has_previous: " << has_previous
+                  << ", has_next: " << has_next << std::endl;
+
+        // Update properties
+        parent->can_play().set(has_tracks);
+        parent->can_pause().set(has_tracks);
+        parent->can_go_previous().set(has_previous);
+        parent->can_go_next().set(has_next);
+    }
+
     // Our link back to our parent.
     media::PlayerImplementation<Parent>* parent;
     // We just store the parameters passed on construction.
@@ -327,8 +349,8 @@ media::PlayerImplementation<Parent>::PlayerImplementation(const media::PlayerImp
       d{std::make_shared<Private>(this, config)}
 {
     // Initialize default values for Player interface properties
-    Parent::can_play().set(true);
-    Parent::can_pause().set(true);
+    Parent::can_play().set(false);
+    Parent::can_pause().set(false);
     Parent::can_seek().set(true);
     Parent::can_go_previous().set(false);
     Parent::can_go_next().set(false);
@@ -354,6 +376,11 @@ media::PlayerImplementation<Parent>::PlayerImplementation(const media::PlayerImp
     };
     Parent::position().install(position_getter);
 
+    d->engine->position().changed().connect([this](uint64_t position)
+    {
+        d->track_list->on_position_changed(position);
+    });
+
     // Make sure that the Duration property gets updated from the Engine
     // every time the client requests duration
     std::function<uint64_t()> duration_getter = [this]()
@@ -376,13 +403,15 @@ media::PlayerImplementation<Parent>::PlayerImplementation(const media::PlayerImp
 
     std::function<bool()> can_go_next_getter = [this]()
     {
-        return d->track_list->has_next();
+        // If LoopStatus == playlist, then there is always a next track
+        return d->track_list->has_next() or Parent::loop_status() != Player::LoopStatus::none;
     };
     Parent::can_go_next().install(can_go_next_getter);
 
     std::function<bool()> can_go_previous_getter = [this]()
     {
-        return d->track_list->has_previous();
+        // If LoopStatus == playlist, then there is always a next previous
+        return d->track_list->has_previous() or Parent::loop_status() != Player::LoopStatus::none;
     };
     Parent::can_go_previous().install(can_go_previous_getter);
 
@@ -484,8 +513,7 @@ media::PlayerImplementation<Parent>::PlayerImplementation(const media::PlayerImp
         }
     });
 
-
-    d->track_list->on_go_to_track().connect([this](std::pair<const media::Track::Id, bool> p)
+    d->track_list->on_go_to_track().connect([this](const media::Track::Id& id)
     {
         // This lambda needs to be mutually exclusive with the about_to_finish lambda above
         const bool locked = d->doing_go_to_track.try_lock();
@@ -494,11 +522,8 @@ media::PlayerImplementation<Parent>::PlayerImplementation(const media::PlayerImp
         if (!locked)
             return;
 
-        const media::Track::Id id = p.first;
-        const bool toggle_player_state = p.second;
-
-        if (toggle_player_state)
-            d->engine->stop();
+        // Store whether we should restore the current playing state after loading the new uri
+        const bool auto_play = Parent::playback_status().get() == media::Player::playing;
 
         const Track::UriType uri = d->track_list->query_uri_for_track(id);
         if (!uri.empty())
@@ -509,8 +534,11 @@ media::PlayerImplementation<Parent>::PlayerImplementation(const media::PlayerImp
             d->engine->open_resource_for_uri(uri, do_pipeline_reset);
         }
 
-        if (toggle_player_state)
+        if (auto_play)
+        {
+            std::cout << "Restoring playing state in on_go_to_track()" << std::endl;
             d->engine->play();
+        }
 
         d->doing_go_to_track.unlock();
     });
@@ -520,8 +548,44 @@ media::PlayerImplementation<Parent>::PlayerImplementation(const media::PlayerImp
         std::cout << "** Track was added, handling in PlayerImplementation" << std::endl;
         if (d->track_list->tracks()->size() == 1)
             d->open_first_track_from_tracklist(id);
+
+        d->update_mpris_properties();
     });
 
+    d->track_list->on_tracks_added().connect([this](const media::TrackList::ContainerURI& tracks)
+    {
+        std::cout << "** Track was added, handling in PlayerImplementation" << std::endl;
+        // If the two sizes are the same, that means the TrackList was previously empty and we need
+        // to open the first track in the TrackList so that is_audio_source() and is_video_source()
+        // will function correctly.
+        if (tracks.size() >= 1 and d->track_list->tracks()->size() == tracks.size())
+            d->open_first_track_from_tracklist(tracks.front());
+
+        d->update_mpris_properties();
+    });
+
+    d->track_list->on_track_removed().connect([this](const media::Track::Id&)
+    {
+        d->update_mpris_properties();
+    });
+
+    d->track_list->on_track_list_reset().connect([this](void)
+    {
+        d->update_mpris_properties();
+    });
+
+    d->track_list->on_track_changed().connect([this](const media::Track::Id&)
+    {
+        d->update_mpris_properties();
+    });
+
+    d->track_list->on_track_list_replaced().connect(
+        [this](const media::TrackList::ContainerTrackIdTuple&)
+        {
+            d->update_mpris_properties();
+        }
+    );
+
     // Everything is setup, we now subscribe to death notifications.
     std::weak_ptr<Private> wp{d};
 
@@ -622,7 +686,15 @@ template<typename Parent>
 bool media::PlayerImplementation<Parent>::open_uri(const Track::UriType& uri)
 {
     d->track_list->reset();
-    const bool ret = d->engine->open_resource_for_uri(uri, false);
+
+    // If empty uri, give the same meaning as QMediaPlayer::setMedia("")
+    if (uri.empty()) {
+        cout << __PRETTY_FUNCTION__ << ": resetting current media" << endl;
+        return true;
+    }
+
+    static const bool do_pipeline_reset = false;
+    const bool ret = d->engine->open_resource_for_uri(uri, do_pipeline_reset);
     // Don't set new track as the current track to play since we're calling open_resource_for_uri above
     static const bool make_current = false;
     d->track_list->add_track_with_uri_at(uri, media::TrackList::after_empty_track(), make_current);
