@@ -55,6 +55,10 @@ public:
     PlayerImplementation(const Configuration& configuration);
     ~PlayerImplementation();
 
+    virtual std::string uuid() const;
+    virtual void reconnect();
+    virtual void abandon();
+
     virtual std::shared_ptr<TrackList> track_list();
     virtual Player::PlayerKey key() const;
 
@@ -78,8 +82,8 @@ private:
     struct Private;
     std::shared_ptr<Private> d;
 };
+
 }
 }
 }
 #endif // CORE_UBUNTU_MEDIA_PLAYER_IMPLEMENTATION_H_
-
