@@ -53,6 +53,8 @@ class PlayerSkeleton : public core::ubuntu::media::Player
     {
         // The bus connection we are associated with.
         std::shared_ptr<core::dbus::Bus> bus;
+        // The service instance we are exposed under.
+        std::shared_ptr<core::dbus::Service> service;
         // The session object that we want to expose the skeleton upon.
         std::shared_ptr<core::dbus::Object> session;
         // Our functional dependencies.
@@ -73,7 +75,7 @@ class PlayerSkeleton : public core::ubuntu::media::Player
     virtual const core::Property<PlaybackStatus>& playback_status() const;
     virtual const core::Property<LoopStatus>& loop_status() const;
     virtual const core::Property<PlaybackRate>& playback_rate() const;
-    virtual const core::Property<bool>& is_shuffle() const;
+    virtual const core::Property<bool>& shuffle() const;
     virtual const core::Property<Track::MetaData>& meta_data_for_current_track() const;
     virtual const core::Property<Volume>& volume() const;
     virtual const core::Property<PlaybackRate>& minimum_playback_rate() const;
@@ -86,7 +88,7 @@ class PlayerSkeleton : public core::ubuntu::media::Player
 
     virtual core::Property<LoopStatus>& loop_status();
     virtual core::Property<PlaybackRate>& playback_rate();
-    virtual core::Property<bool>& is_shuffle();
+    virtual core::Property<bool>& shuffle();
     virtual core::Property<Volume>& volume();
     virtual core::Property<AudioStreamRole>& audio_stream_role();
     virtual core::Property<Lifetime>& lifetime();
