@@ -39,10 +39,16 @@ class PlayerStub : public Player
   public:
     explicit PlayerStub(
         const std::shared_ptr<Service>& parent,
-        const std::shared_ptr<core::dbus::Object>& object);
+        const std::shared_ptr<core::dbus::Service>& service,
+        const std::shared_ptr<core::dbus::Object>& object,
+        const std::string& uuid = std::string{});
 
     ~PlayerStub();
 
+    virtual std::string uuid() const;
+    virtual void reconnect();
+    virtual void abandon();
+
     virtual std::shared_ptr<TrackList> track_list();
     virtual PlayerKey key() const;
 
@@ -67,7 +73,7 @@ class PlayerStub : public Player
     virtual const core::Property<PlaybackStatus>& playback_status() const;
     virtual const core::Property<LoopStatus>& loop_status() const;
     virtual const core::Property<PlaybackRate>& playback_rate() const;
-    virtual const core::Property<bool>& is_shuffle() const;
+    virtual const core::Property<bool>& shuffle() const;
     virtual const core::Property<Track::MetaData>& meta_data_for_current_track() const;
     virtual const core::Property<Volume>& volume() const;
     virtual const core::Property<PlaybackRate>& minimum_playback_rate() const;
@@ -80,7 +86,7 @@ class PlayerStub : public Player
 
     virtual core::Property<LoopStatus>& loop_status();
     virtual core::Property<PlaybackRate>& playback_rate();
-    virtual core::Property<bool>& is_shuffle();
+    virtual core::Property<bool>& shuffle();
     virtual core::Property<Volume>& volume();
     virtual core::Property<AudioStreamRole>& audio_stream_role();
     virtual core::Property<Lifetime>& lifetime();
