@@ -58,7 +58,7 @@ media::ServiceStub::~ServiceStub()
 std::shared_ptr<media::Player> media::ServiceStub::create_session(const media::Player::Configuration&)
 {
     auto op = d->object->invoke_method_synchronously<mpris::Service::CreateSession,
-         dbus::types::ObjectPath>();
+         std::tuple<dbus::types::ObjectPath, std::string>>();
 
     if (op.is_error())
         throw std::runtime_error("Problem creating session: " + op.error());
@@ -66,10 +66,47 @@ std::shared_ptr<media::Player> media::ServiceStub::create_session(const media::P
     return std::shared_ptr<media::Player>(new media::PlayerStub
     {
         shared_from_this(),
-        access_service()->object_for_path(op.value())
+        access_service(),
+        access_service()->object_for_path(std::get<0>(op.value())),
+        std::get<1>(op.value())
     });
 }
 
+void media::ServiceStub::detach_session(const std::string& uuid, const media::Player::Configuration&)
+{
+    auto op = d->object->invoke_method_synchronously<mpris::Service::DetachSession,
+         void>(uuid);
+
+    if (op.is_error())
+        throw std::runtime_error("Problem detaching session: " + op.error());
+}
+
+std::shared_ptr<media::Player> media::ServiceStub::reattach_session(const std::string& uuid, const media::Player::Configuration&)
+{
+    auto op = d->object->invoke_method_synchronously<mpris::Service::ReattachSession,
+         dbus::types::ObjectPath>(uuid);
+
+    if (op.is_error())
+        throw std::runtime_error("Problem reattaching session: " + op.error());
+
+    return std::shared_ptr<media::Player>(new media::PlayerStub
+    {
+        shared_from_this(),
+        access_service(),
+        access_service()->object_for_path(op.value()),
+        uuid
+    });
+}
+
+void media::ServiceStub::destroy_session(const std::string& uuid, const media::Player::Configuration&)
+{
+    auto op = d->object->invoke_method_synchronously<mpris::Service::DestroySession,
+         void>(uuid);
+
+    if (op.is_error())
+        throw std::runtime_error("Problem destroying session: " + op.error());
+}
+
 std::shared_ptr<media::Player> media::ServiceStub::create_fixed_session(const std::string& name, const media::Player::Configuration&)
 {
     auto op = d->object->invoke_method_synchronously<mpris::Service::CreateFixedSession,
@@ -81,6 +118,7 @@ std::shared_ptr<media::Player> media::ServiceStub::create_fixed_session(const st
     return std::shared_ptr<media::Player>(new media::PlayerStub
     {
         shared_from_this(),
+        access_service(),
         access_service()->object_for_path(op.value())
     });
 }
@@ -96,13 +134,22 @@ std::shared_ptr<media::Player> media::ServiceStub::resume_session(media::Player:
     return std::shared_ptr<media::Player>(new media::PlayerStub
     {
         shared_from_this(),
+        access_service(),
         access_service()->object_for_path(op.value())
     });
 }
 
+void media::ServiceStub::set_current_player(Player::PlayerKey key)
+{
+    auto op = d->object->invoke_method_synchronously<mpris::Service::SetCurrentPlayer,
+         void>(key);
+
+    if (op.is_error())
+        throw std::runtime_error("Problem setting current player: " + op.error());
+}
+
 void media::ServiceStub::pause_other_sessions(media::Player::PlayerKey key)
 {
-    std::cout << __PRETTY_FUNCTION__ << std::endl;
     auto op = d->object->invoke_method_synchronously<mpris::Service::PauseOtherSessions,
          void>(key);
 
