@@ -25,6 +25,7 @@
 #include "track_list_traits.h"
 #include "the_session_bus.h"
 
+#include "mpris/player.h"
 #include "mpris/track_list.h"
 
 #include <core/dbus/property.h>
@@ -52,7 +53,10 @@ struct media::TrackListStub::Private
           signals
           {
               object->get_signal<mpris::TrackList::Signals::TrackAdded>(),
+              object->get_signal<mpris::TrackList::Signals::TracksAdded>(),
+              object->get_signal<mpris::TrackList::Signals::TrackMoved>(),
               object->get_signal<mpris::TrackList::Signals::TrackRemoved>(),
+              object->get_signal<mpris::TrackList::Signals::TrackListReset>(),
               object->get_signal<mpris::TrackList::Signals::TrackListReplaced>(),
               object->get_signal<mpris::TrackList::Signals::TrackChanged>()
           }
@@ -70,22 +74,37 @@ struct media::TrackListStub::Private
     struct Signals
     {
         typedef core::dbus::Signal<mpris::TrackList::Signals::TrackAdded, mpris::TrackList::Signals::TrackAdded::ArgumentType> DBusTrackAddedSignal;
+        typedef core::dbus::Signal<mpris::TrackList::Signals::TracksAdded, mpris::TrackList::Signals::TracksAdded::ArgumentType> DBusTracksAddedSignal;
+        typedef core::dbus::Signal<mpris::TrackList::Signals::TrackMoved, mpris::TrackList::Signals::TrackMoved::ArgumentType> DBusTrackMovedSignal;
         typedef core::dbus::Signal<mpris::TrackList::Signals::TrackRemoved, mpris::TrackList::Signals::TrackRemoved::ArgumentType> DBusTrackRemovedSignal;
+        typedef core::dbus::Signal<
+            mpris::TrackList::Signals::TrackListReset,
+            mpris::TrackList::Signals::TrackListReset::ArgumentType>
+                DBusTrackListResetSignal;
         typedef core::dbus::Signal<mpris::TrackList::Signals::TrackListReplaced, mpris::TrackList::Signals::TrackListReplaced::ArgumentType> DBusTrackListReplacedSignal;
         typedef core::dbus::Signal<mpris::TrackList::Signals::TrackChanged, mpris::TrackList::Signals::TrackChanged::ArgumentType> DBusTrackChangedSignal;
 
         Signals(const std::shared_ptr<DBusTrackAddedSignal>& track_added,
+                const std::shared_ptr<DBusTracksAddedSignal>& tracks_added,
+                const std::shared_ptr<DBusTrackMovedSignal>& track_moved,
                 const std::shared_ptr<DBusTrackRemovedSignal>& track_removed,
+                const std::shared_ptr<DBusTrackListResetSignal>& track_list_reset,
                 const std::shared_ptr<DBusTrackListReplacedSignal>& track_list_replaced,
                 const std::shared_ptr<DBusTrackChangedSignal>& track_changed)
             : on_track_added(),
+              on_tracks_added(),
+              on_track_moved(),
               on_track_removed(),
+              on_track_list_reset(),
               on_track_list_replaced(),
               on_track_changed(),
               dbus
               {
                   track_added,
+                  tracks_added,
+                  track_moved,
                   track_removed,
+                  track_list_reset,
                   track_list_replaced,
                   track_changed,
               }
@@ -96,15 +115,33 @@ struct media::TrackListStub::Private
                 on_track_added(id);
             });
 
+            dbus.on_tracks_added->connect([this](const media::TrackList::ContainerURI& tracks)
+            {
+                std::cout << "OnTracksAdded signal arrived via the bus." << std::endl;
+                on_tracks_added(tracks);
+            });
+
+            dbus.on_track_moved->connect([this](const media::TrackList::TrackIdTuple& ids)
+            {
+                std::cout << "OnTrackMoved signal arrived via the bus." << std::endl;
+                on_track_moved(ids);
+            });
+
             dbus.on_track_removed->connect([this](const Track::Id& id)
             {
                 std::cout << "OnTrackRemoved signal arrived via the bus." << std::endl;
                 on_track_removed(id);
             });
 
+            dbus.on_track_list_reset->connect([this](void)
+            {
+                std::cout << "OnTrackListReset signal arrived via the bus." << std::endl;
+                on_track_list_reset();
+            });
+
             dbus.on_track_list_replaced->connect([this](const media::TrackList::ContainerTrackIdTuple& list)
             {
-                std::cout << "OnTrackListRemoved signal arrived via the bus." << std::endl;
+                std::cout << "OnTrackListReplaced signal arrived via the bus." << std::endl;
                 on_track_list_replaced(list);
             });
 
@@ -116,16 +153,22 @@ struct media::TrackListStub::Private
         }
 
         core::Signal<Track::Id> on_track_added;
+        core::Signal<media::TrackList::ContainerURI> on_tracks_added;
+        core::Signal<media::TrackList::TrackIdTuple> on_track_moved;
         core::Signal<Track::Id> on_track_removed;
+        core::Signal<void> on_track_list_reset;
         core::Signal<media::TrackList::ContainerTrackIdTuple> on_track_list_replaced;
         core::Signal<Track::Id> on_track_changed;
-        core::Signal<std::pair<Track::Id, bool>> on_go_to_track;
+        core::Signal<Track::Id> on_go_to_track;
         core::Signal<void> on_end_of_tracklist;
 
         struct DBus
         {
             std::shared_ptr<DBusTrackAddedSignal> on_track_added;
+            std::shared_ptr<DBusTracksAddedSignal> on_tracks_added;
+            std::shared_ptr<DBusTrackMovedSignal> on_track_moved;
             std::shared_ptr<DBusTrackRemovedSignal> on_track_removed;
+            std::shared_ptr<DBusTrackListResetSignal> on_track_list_reset;
             std::shared_ptr<DBusTrackListReplacedSignal> on_track_list_replaced;
             std::shared_ptr<DBusTrackChangedSignal> on_track_changed;
         } dbus;
@@ -184,16 +227,66 @@ media::Track::UriType media::TrackListStub::query_uri_for_track(const media::Tra
 
 void media::TrackListStub::add_track_with_uri_at(
         const media::Track::UriType& uri,
-        const media::Track::Id& id,
+        const media::Track::Id& position,
         bool make_current)
 {
     auto op = d->object->invoke_method_synchronously<mpris::TrackList::AddTrack, void>(
                 uri,
-                id,
+                position,
                 make_current);
 
     if (op.is_error())
-        throw std::runtime_error("Problem adding track: " + op.error());
+    {
+        if (op.error().name() ==
+                mpris::TrackList::Error::InsufficientPermissionsToAddTrack::name)
+            throw media::TrackList::Errors::InsufficientPermissionsToAddTrack{};
+        else if (op.error().name() == mpris::Player::Error::UriNotFound::name)
+            throw media::Player::Errors::UriNotFound{op.error().print()};
+        else
+            throw std::runtime_error{op.error().print()};
+    }
+}
+
+void media::TrackListStub::add_tracks_with_uri_at(const ContainerURI& uris, const Track::Id& position)
+{
+    auto op = d->object->invoke_method_synchronously<mpris::TrackList::AddTracks, void>(
+                uris,
+                position);
+
+    if (op.is_error())
+    {
+        if (op.error().name() ==
+                mpris::TrackList::Error::InsufficientPermissionsToAddTrack::name)
+            throw media::TrackList::Errors::InsufficientPermissionsToAddTrack{};
+        else if (op.error().name() == mpris::Player::Error::UriNotFound::name)
+            throw media::Player::Errors::UriNotFound{op.error().print()};
+        else
+            throw std::runtime_error{op.error().print()};
+    }
+}
+
+bool media::TrackListStub::move_track(const media::Track::Id& id, const media::Track::Id& to)
+{
+    auto op = d->object->invoke_method_synchronously<mpris::TrackList::MoveTrack, void>(id, to);
+
+    if (op.is_error())
+    {
+        if (op.error().name() ==
+                mpris::TrackList::Error::FailedToMoveTrack::name)
+            throw media::TrackList::Errors::FailedToMoveTrack{};
+        else if (op.error().name() ==
+                mpris::TrackList::Error::FailedToFindMoveTrackSource::name)
+            throw media::TrackList::Errors::FailedToFindMoveTrackSource{op.error().print()};
+        else if (op.error().name() ==
+                mpris::TrackList::Error::FailedToFindMoveTrackDest::name)
+            throw media::TrackList::Errors::FailedToFindMoveTrackDest{op.error().print()};
+        else
+            throw std::runtime_error{op.error().print()};
+
+        return false;
+    }
+
+    return true;
 }
 
 void media::TrackListStub::remove_track(const media::Track::Id& track)
@@ -202,14 +295,18 @@ void media::TrackListStub::remove_track(const media::Track::Id& track)
                 track);
 
     if (op.is_error())
-        throw std::runtime_error("Problem removing track: " + op.error());
+    {
+        if (op.error().name() ==
+                mpris::TrackList::Error::TrackNotFound::name)
+            throw media::TrackList::Errors::TrackNotFound{};
+        else
+            throw std::runtime_error{"Problem removing track: " + op.error().print()};
+    }
 }
 
-void media::TrackListStub::go_to(const media::Track::Id& track, bool toggle_player_state)
+void media::TrackListStub::go_to(const media::Track::Id& track)
 {
-    (void) toggle_player_state;
-    auto op = d->object->invoke_method_synchronously<mpris::TrackList::GoTo, void>(
-                track);
+    auto op = d->object->invoke_method_synchronously<mpris::TrackList::GoTo, void>(track);
 
     if (op.is_error())
         throw std::runtime_error("Problem adding track: " + op.error());
@@ -227,16 +324,6 @@ media::Track::Id media::TrackListStub::previous()
     return media::Track::Id{"/empty/track/id"};
 }
 
-void media::TrackListStub::shuffle_tracks()
-{
-    std::cerr << "shuffle_tracks() does nothing from the client side" << std::endl;
-}
-
-void media::TrackListStub::unshuffle_tracks()
-{
-    std::cerr << "unshuffle_tracks() does nothing from the client side" << std::endl;
-}
-
 void media::TrackListStub::reset()
 {
     auto op = d->object->invoke_method_synchronously<mpris::TrackList::Reset, void>();
@@ -255,17 +342,32 @@ const core::Signal<media::Track::Id>& media::TrackListStub::on_track_added() con
     return d->signals.on_track_added;
 }
 
+const core::Signal<media::TrackList::ContainerURI>& media::TrackListStub::on_tracks_added() const
+{
+    return d->signals.on_tracks_added;
+}
+
+const core::Signal<media::TrackList::TrackIdTuple>& media::TrackListStub::on_track_moved() const
+{
+    return d->signals.on_track_moved;
+}
+
 const core::Signal<media::Track::Id>& media::TrackListStub::on_track_removed() const
 {
     return d->signals.on_track_removed;
 }
 
+const core::Signal<void>& media::TrackListStub::on_track_list_reset() const
+{
+    return d->signals.on_track_list_reset;
+}
+
 const core::Signal<media::Track::Id>& media::TrackListStub::on_track_changed() const
 {
     return d->signals.on_track_changed;
 }
 
-const core::Signal<std::pair<media::Track::Id, bool>>& media::TrackListStub::on_go_to_track() const
+const core::Signal<media::Track::Id>& media::TrackListStub::on_go_to_track() const
 {
     return d->signals.on_go_to_track;
 }
